/**
 * COP4331 Group18 TankWars
 * @author Ryan Farneski
 * @author John Gilmore
 * @author Thomas Hellinger
 * @author Edward Jezisek
 * @author Brandon Ramsey
 * @author Andrew Watson
 * Tank.java
 * @version 1.0
 * Mar 13, 2013
 * Description: Environment which draws all the tanks, the bullets, etc..
 * In Progress: Need to implement collision detection.
 */
package com.amphibian.environment;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Random;

import com.amphibian.player.*;
import com.amphibian.tank.Armament;
import com.amphibian.tank.Tank;
import com.amphibian.tankwars.Bitmaps;
import com.amphibian.tankwars.R;

import android.content.Context;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.graphics.Canvas;
import android.graphics.Color;
import android.graphics.Matrix;
import android.graphics.Paint;
import android.graphics.RectF;
import android.util.DisplayMetrics;

public class Environment {
    Bitmap bitmap;
    Paint paint;
    final int tankWidth = 50;
    final int tankHeight = 25;
    final int SCREEN_MAX = 500;
    final int SCREEN_MIN = 0;
    public static double GRAVITY = 1.0;
    final Bitmap.Config conf = Bitmap.Config.ARGB_8888;
    public RectF lScreenBorder = new RectF(0, 0, 0, 0);
    public RectF rScreenBorder = new RectF(0, 0, 0, 0);
    private Bitmap curr;
    public static ArrayList<HumanPlayer> humanPlayers; //TODO Should this be an array?
    private ArrayList<AI_Player> computerPlayers;
    public static Armament weaponInPlay; 
    public static double timeInFlight; //TODO make private?
    public static boolean isShotRight;
    private Bitmap turretLeft;
    private Bitmap turretRight;
    private Bitmap bullet;
    private Canvas canvas;
    private Context context;
    
    
    /**
     * @param context
     * @param screenWidth
     * @param screenHeight
     */
    public Environment(Context context, int screenWidth, int screenHeight) {
            this.context = context;
            paint = new Paint();
            paint.setAntiAlias(true);
            curr = Bitmap.createBitmap(screenWidth, screenHeight, conf);
            canvas = new Canvas(curr);
            //TODO next two lines need to be input from user, not hardcoded 
            humanPlayers = new ArrayList<HumanPlayer>(2);
            this.computerPlayers = null;
            this.initializeTanks(2);
            this.shufflePlayers(); //Must follow initializeTanks
            this.refreshEnvironment();
            lScreenBorder = new RectF(0, 0, 1, 300);
            rScreenBorder = new RectF(499,0,500,300);
            Environment.timeInFlight = 0;
        }
    
    /**
     * Randomly shuffles the order players take turns in.
     * @author Thomas Adriaan Hellinger
     */
    private void shufflePlayers() {
    	long seed = System.nanoTime();
    	Collections.shuffle(humanPlayers, new Random(seed));
    }
    
    public Armament getWeaponInPlay(){ return Environment.weaponInPlay; }
    
    public ArrayList<HumanPlayer> getActiveHumanPlayers() { return humanPlayers; }
    
    public ArrayList<AI_Player> getActiveComputerPlayers() { return this.computerPlayers; }
    
    /**
     * @author Thomas Adriaan Hellinger
     * @param currentPlayer
     * @param activeTank
     * @param isRight
     * @return
     */
    public Collision checkForCollisions(HumanPlayer currentPlayer, Tank activeTank, boolean isRight){
    	for(HumanPlayer otherPlayer : Environment.humanPlayers){
    		if(currentPlayer.equals(otherPlayer)) continue;
    		Tank otherTank = otherPlayer.get_controlled_tank();
            if (activeTank.rect.intersect
                    (otherTank.getRect())) {
            	if(isRight){ //Rightward ram into other tank
            		activeTank.rect.set(otherTank.rect.left-51, activeTank.rect.top, otherTank.rect.left-1, activeTank.rect.bottom);
            		activeTank.setPosition((int)otherTank.rect.left-50, activeTank.getPosY());
            	} else { //leftward ram into other tank
            		activeTank.rect.set(otherTank.rect.right+1, activeTank.rect.top, otherTank.rect.right+51, activeTank.rect.bottom);
            		activeTank.setPosition((int)otherTank.rect.right+1, activeTank.getPosY());
            	}
            	//TODO add code for ramming
                return Collision.TANK;
            }
            else if (activeTank.getRect().intersect
                    (lScreenBorder)) {
            	if(isRight) return Collision.NONE;
            	else return Collision.BORDER;
            }
            else if (activeTank.getRect().intersect
                    (rScreenBorder)) {
            	if(!isRight) return Collision.NONE;
            	else return Collision.BORDER;
            }
            
            else continue;
    	}
    	return Collision.NONE;
    }

    private void initializeTanks(int numberOfPlayers) {
    	Bitmap tankRight;
    	Bitmap tankLeft; //TODO Rename these, make for loop for initializing multiple tanks
        //This creates the right tank bitmap.
        tankRight = BitmapFactory.decodeResource(context.getResources(), 
                                                    R.drawable.tank);
        tankRight = Bitmaps.resizeBitmap(tankRight, tankWidth, tankHeight);
        
        //This creates  the right turret.
        turretRight = BitmapFactory.decodeResource(context.getResources(),
                                                    R.drawable.turret);
        turretRight = Bitmaps.resizeBitmap
                                (turretRight, tankWidth / 2, tankHeight / 5);
        Matrix m = new Matrix();
        m.preScale(-1, 1);
        
        //This rotates the right tank bitmap and makes it the left tank.
        tankLeft = Bitmap.createBitmap(tankRight, 0, 0, 
                tankRight.getWidth(), tankRight.getHeight(),m, false);
        tankLeft = Bitmaps.resizeBitmap(tankLeft, tankWidth, tankHeight);
        
        //This rotates the right turret bitmap and makes the left turret.
        turretLeft = Bitmap.createBitmap(turretRight, 0, 0, 
                turretRight.getWidth(), turretRight.getHeight(),m, false);
        turretLeft.setDensity(DisplayMetrics.DENSITY_DEFAULT);
        
        bullet = BitmapFactory.decodeResource(context.getResources(),
                            R.drawable.bullet);
        bullet = Bitmaps.resizeBitmap(bullet, bullet.getHeight() / 7,
                            bullet.getWidth() / 7);
        
        //Create the first tank
        Tank tankOne = new Tank(100, true, tankLeft);
        //Create the second tank
        Tank tankTwo = new Tank(400, false, tankRight);
        
      //TODO need for loop for initializing players, may need to put in Environment constructor
    	HumanPlayer playerOne = new HumanPlayer(0, tankOne, "Patton");
    	HumanPlayer playerTwo = new HumanPlayer(0, tankTwo, "Rommel");
    	
    	//Add all players to list of active players        
        humanPlayers.add(playerOne); //TODO change as part of initialize multiple tank for loop
        humanPlayers.add(playerTwo);
    }
    
    public void refreshEnvironment() {
        paint.setColor(Color.CYAN);
        canvas.drawRect(0, 150, 500, 0, paint);
        paint.setColor(Color.GREEN);
        canvas.drawRect(0, 300, 500, 150, paint);
    }
    
    public void drawTank(Tank aTank) {
    	canvas.drawBitmap(aTank.sprite, aTank.getPosX(), 150 - tankHeight, paint);
        drawTurret(aTank);
    }
    
    public void drawTurret(Tank aTank) {
        Matrix matrix = new Matrix();
    
        if(aTank.getRotate()) {
            float xPos = aTank.getPosX() + aTank.sprite.getWidth() - 20;
            float yPos = (float) (150 - .75 * tankHeight);
            matrix.setRotate(-aTank.turret.getAngle(), 0, 0);
            matrix.postTranslate(xPos, yPos);
            canvas.drawBitmap(turretLeft, matrix, paint);
        }
        else {
            float xPos = aTank.getPosX() - 5;
            float yPos = (float) (150 - .75 *tankHeight);
            matrix.setRotate(aTank.turret.getAngle(), turretRight.getWidth(), 0);
            matrix.postTranslate(xPos, yPos);
            canvas.drawBitmap(turretRight, matrix, paint);
        }
    }
    
    public void drawTankHitBox(Tank aTank) { //TODO Should tank drawing fn be in Tank?
        paint.setColor(Color.RED);
        canvas.drawRect(aTank.rect, paint);
    }
    
    public void drawBulletHitBox() {
        paint.setColor(Color.RED);
        canvas.drawRect(weaponInPlay.bulHitBox, paint);
    }
    
    public Bitmap getBitmap() {
        return curr;
    }
    
    public void drawBullet(float xPos, float yPos) {
    	
    	weaponInPlay.bulHitBox.set(xPos - 5, yPos + 5, xPos + 5, yPos - 5);
//        canvas.drawBitmap(bullet, xPos, yPos, paint);
        paint.setColor(Color.BLACK);
        
        canvas.drawCircle(xPos, yPos, 5, paint);
//        drawBulletHitBox();
    }
    
    public void removeHumanPlayer(HumanPlayer deadman){
    	humanPlayers.remove(deadman);
    }
    
    public void removeComputerPlayer(AI_Player deadman){
    	this.computerPlayers.remove(deadman);
    }
    
    public void placeTanks(){
    	
    }

}
